from easyenvi.file import format_converter

loader_config = {
    'csv':      format_converter.csv_loader,
    'docx':     format_converter.docx_loader,
    'jpg':      format_converter.jpg_loader,
    'json':     format_converter.json_loader,
    'md':       format_converter.md_loader,
    'parquet':  format_converter.parquet_loader,
    'pdf':      format_converter.pdf_loader,
    'pickle':   format_converter.pickle_loader,
    'png':      format_converter.png_loader,
    'pptx':     format_converter.pptx_loader,
    'sql':      format_converter.sql_loader,
    'toml':     format_converter.toml_loader,
    'txt':      format_converter.txt_loader,
    'xlsx':     format_converter.xlsx_loader,
    'xml':      format_converter.xml_loader,
    'yaml':     format_converter.yaml_loader,
    'yml':      format_converter.yaml_loader
}

saver_config = {
    'csv':      format_converter.csv_saver,
    'docx':     format_converter.docx_saver,
    'jpg':      format_converter.jpg_saver,
    'json':     format_converter.json_saver,
    'md':       format_converter.md_saver,
    'parquet':  format_converter.parquet_saver,
    'pdf':      format_converter.pdf_saver,
    'pickle':   format_converter.pickle_saver,
    'png':      format_converter.png_saver,
    'pptx':     format_converter.pptx_saver,
    'sql':      format_converter.sql_saver,
    'toml':     format_converter.toml_saver,
    'txt':      format_converter.txt_saver,
    'xlsx':     format_converter.xlsx_saver,
    'xml':      format_converter.xml_saver,
    'yaml':     format_converter.yaml_saver,
    'yml':      format_converter.yaml_saver
}

def load(path, **kwargs):

    extension = path.split('.')[-1]

    if extension not in loader_config:
        raise ValueError(f"Extension '{extension}' is not currently supported.")

    loader = loader_config[extension]

    return loader(path, **kwargs)

def save(obj, path, **kwargs):

    extension = path.split('.')[-1]

    if extension not in saver_config:
        raise ValueError(f"Extension '{extension}' is not currently supported.")

    saver = saver_config[extension]

    return saver(obj, path, **kwargs)


import copy
import json
import pickle
import toml
import xml.etree.ElementTree as ET
import yaml

import fsspec
import pandas as pd

from docx import Document
from PIL import Image
from pptx import Presentation
from PyPDF2 import PdfReader, PdfWriter

# CSV
def csv_loader(path, **kwargs):
    with fsspec.open(path, 'rb', **kwargs) as f:
        return pd.read_csv(f)

def csv_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wb', **kwargs) as f:
        obj.to_csv(f)

# DOCX
def docx_loader(path, **kwargs):
    with fsspec.open(path, 'rb', **kwargs) as f:
        return Document(f)

def docx_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wb', **kwargs) as f:
        obj.save(f)

# JPG
def jpg_loader(path, **kwargs):
    with fsspec.open(path, 'rb', **kwargs) as f:
        return Image.open(f).copy()

def jpg_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wb', **kwargs) as f:
        obj.save(f)

# JSON
def json_loader(path, **kwargs):
    with fsspec.open(path, 'rt', **kwargs) as f:
        return json.load(f)

def json_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wt', **kwargs) as f:
        json.dump(obj, f)

# MD
def md_loader(path, **kwargs):
    with fsspec.open(path, 'rt', **kwargs) as f:
        return f.read()

def md_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wt', **kwargs) as f:
        f.write(obj)

# PARQUET
def parquet_loader(path, **kwargs):
    with fsspec.open(path, 'rb', **kwargs) as f:
        return pd.read_parquet(f)

def parquet_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wb', **kwargs) as f:
        obj.to_parquet(f)

# PDF
def pdf_loader(path, **kwargs):
    with fsspec.open(path, 'rb', **kwargs) as f:
        return copy.deepcopy(PdfReader(f))

def pdf_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wb', **kwargs) as f:
        output = PdfWriter()
        for page in obj.pages:
            output.add_page(page)
    
        output.write(f)

# PICKLE
def pickle_loader(path, **kwargs):
    with fsspec.open(path, 'rb', **kwargs) as f:
        return pickle.load(f)

def pickle_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wb', **kwargs) as f:
        pickle.dump(obj, f, protocol=pickle.HIGHEST_PROTOCOL)

# PPTX
def pptx_loader(path, **kwargs):
    with fsspec.open(path, 'rb', **kwargs) as f:
        return Presentation(f)

def pptx_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wb', **kwargs) as f:
        obj.save(f)

# PNG
def png_loader(path, **kwargs):
    with fsspec.open(path, 'rb', **kwargs) as f:
        return Image.open(f).copy()

def png_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wb', **kwargs) as f:
        obj.save(f)

# SQL
def sql_loader(path, **kwargs):
    with fsspec.open(path, 'rt', **kwargs) as f:
        return f.read()

def sql_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wt', **kwargs) as f:
        f.write(obj)

# TOML
def toml_loader(path, **kwargs):
    with fsspec.open(path, 'rt', **kwargs) as f:
        return toml.load(f)

def toml_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wt', **kwargs) as f:
        toml.dump(obj, f)

# TXT
def txt_loader(path, **kwargs):
    with fsspec.open(path, 'rt', **kwargs) as f:
        return f.read()

def txt_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wt', **kwargs) as f:
        f.write(obj)

# XML
def xml_loader(path, **kwargs):
    with fsspec.open(path, 'rb', **kwargs) as f:
        tree = ET.parse(f)
        root = tree.getroot()
        return root

def xml_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wb', **kwargs) as f:
        tree = ET.ElementTree(obj)
        tree.write(f)

# XLSX
def xlsx_loader(path, **kwargs):
    with fsspec.open(path, 'rb', **kwargs) as f:
        return pd.read_excel(f)

def xlsx_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wb', **kwargs) as f:
        obj.to_excel(f)

# YAML
def yaml_loader(path, **kwargs):
    with fsspec.open(path, 'rt', **kwargs) as f:
        return yaml.safe_load(f)

def yaml_saver(obj, path, **kwargs):
    with fsspec.open(path, 'wt', **kwargs) as f:
        yaml.dump(obj, f)


from easyenvi.envs.sharepoint import sharepoint
from easyenvi.envs.gcloud import gcloud
from easyenvi.envs.disk import disk

class EasyEnvironment:
    """
    Easy-to-use functionality for managing files and data in different environments (local, google cloud, sharepoint).
    
    Parameters
    ----------
    local_path : str (optional)
        The path from which interactions with the local environment will be performed.
    gcloud_project_id : str (optional)
        The ID of the Google Cloud project.
    gcloud_credential_path : str (optional)
        The path to the .json Google Cloud credentials file.
    GCS_path : str (optional)
        The path from which interactions with Google Cloud Storage environment will be performed.
    sharepoint_site_url : str (optional)
        SharePoint site.
    sharepoint_client_id : str (optional)
        Client ID.
    sharepoint_client_secret : str (optional)
        Client secret.
    sharepoint_username : str (optional)
        User name of a SharePoint user account.
    sharepoint_user_password : str (optional)
        User password of a SharePoint user account.
    extra_loader_config : dict (optional)
        Extra configuration for file loaders.
    extra_saver_config : dict (optional)
        Extra configuration for file savers.

    Notes
    -----
    Multi-extension management: 
        To learn more about the extensions supported by default, refer to the documentation : https://antoinepinto.gitbook.io/easy-environment/
        To integrate other extensions into the tool, see documentation "Customise supported formats": https://antoinepinto.gitbook.io/easy-environment/extra/customise-supported-formats
    SharePoint environment:
        Need to obtain credentials: https://learn.microsoft.com/en-us/sharepoint/dev/solution-guidance/security-apponly-azureacs
        Either the pair client_id - client_secret is required, either the pair username - user_password
    """

    def __init__(self, local_path="", gcloud_project_id=None, gcloud_credential_path=None,
                 GCS_path=None, sharepoint_site_url=None, sharepoint_client_id=None,
                 sharepoint_client_secret=None, sharepoint_username=None, 
                 sharepoint_user_password=None, extra_loader_config=None, extra_saver_config=None):
    
        self.local = disk(
            root_path=local_path, 
            extra_loader_config=extra_loader_config, 
            extra_saver_config=extra_saver_config)

        if gcloud_project_id is not None:

            self.gcloud = gcloud(
                project_id=gcloud_project_id, 
                GCS_path=GCS_path, 
                credential_path=gcloud_credential_path,
                extra_loader_config=extra_loader_config, 
                extra_saver_config=extra_saver_config)
            
        if sharepoint_site_url is not None:

            self.sharepoint = sharepoint(
                site_url=sharepoint_site_url,
                client_id=sharepoint_client_id,
                client_secret=sharepoint_client_secret,
                username=sharepoint_username, 
                user_password=sharepoint_user_password
                        )


with open("README.md", "r") as f:
    long_description = f.read()

from setuptools import setup, find_packages

setup(
    name='easyenvi',
    version='1.0.6',
    description='Easy-to-use functionality for managing files and data in different environments',
    author='Antoine PINTO',
    author_email='antoine.pinto1@outlook.fr',
    license='MIT',
    license_file='LICENSE',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/AntoinePinto/easyenvi',
    project_urls={
        'Documentation': 'https://antoinepinto.gitbook.io/easyenvi/',
        'Source Code': 'https://github.com/AntoinePinto/easyenvi',
    },
    keywords=["file-manager", "easy-to-use", "environment", "cloud",
              "gcp", "sharepoint"],
    packages=find_packages(),
    install_requires=[
        "db-dtypes>=1.2.0",
        "fsspec>=2023.1.0",
        "gcsfs>=2023.1.0",
        "google-cloud-bigquery>=3.14.1",
        "google-cloud-storage>=2.14.0",
        "Office365-REST-Python-Client>=2.5.4",
        "openpyxl>=3.1.2",
        "pandas>=1.3.5",
        "pillow>=9.5.0",
        "pyarrow>=12.0.1",
        "PyPDF2>=3.0.1",
        "python-docx>=1.1.0",
        "python-pptx>=0.6.23",
        "pyyaml>=6.0.1",
        "toml>=0.10.2"
    ],
    python_requires='>=3.7'
)


import os

from easyenvi import file

class disk:
    """
    Allows interaction with local environment.

    Parameters
    ----------
    local_path : str
        The path from which interactions with the local environment will be performed.
    extra_loader_config : dict
        Extra configuration for file loaders. Default is None.
    extra_saver_config :dict
        Extra configuration for file savers. Default is None.
    """

    def __init__(self, root_path, extra_loader_config=None, extra_saver_config=None):
        self.root_path = root_path
        
    def load(self, path, **kwargs):
        """
        Load a file.
        To learn more about the extensions supported by default, refer to the documentation : https://antoinepinto.gitbook.io/easy-environment/
        To integrate other extensions into the tool, see documentation "Customise supported formats": https://antoinepinto.gitbook.io/easy-environment/extra/customise-supported-formats

        Parameters
        ----------
        path : str
            path to load from.
        """

        load_path = os.path.join(self.root_path, path)
        return file.load(load_path)

    def save(self, obj, path, **kwargs):
        """
        Save a file
        To learn more about the extensions supported by default, refer to the documentation : https://antoinepinto.gitbook.io/easy-environment/
        To integrate other extensions into the tool, see documentation "Customise supported formats": https://antoinepinto.gitbook.io/easy-environment/extra/customise-supported-formats

        Parameters
        ----------
        obj
            object to save.
        path : str
            path to save to.
        """

        save_path = os.path.join(self.root_path, path)
        return file.save(obj, save_path)

    def clear_folder(self, path):
        """
        Clear a folder
        
        Parameters
        ----------
        path : str
            path at which to delete all files.
        """
        folder_path = os.path.join(self.root_path, path)
        for filename in os.listdir(folder_path):
            file_path = os.path.join(folder_path, filename)
            os.remove(file_path)